@@ -12,8 +12,7 @@
 #define EN_NAME "Services"
 #define FR_NAME "Services"
 
-#define EN_FAMILY "Misc."
-#define FR_FAMILY "Divers"
+#define EN_FAMILY "Service detection"
 
 #define EN_DESC "This plugin attempts to guess which\n\
 service is running on the remote ports. For instance,\n\
@@ -62,7 +61,7 @@ int plugin_init(desc)
  struct arglist * desc;
 { 
  plug_set_id(desc, 10330);
- plug_set_version(desc, "$Revision: 1.200 $");
+ plug_set_version(desc, "$Revision: 1.225 $");
  
  plug_set_name(desc, FR_NAME, "francais");
  plug_set_name(desc, EN_NAME, NULL);
@@ -71,7 +70,9 @@ int plugin_init(desc)
  plug_set_category(desc, ACT_GATHER_INFO);
  
  
+#ifdef FR_FAMILY
  plug_set_family(desc, FR_FAMILY, "francais");
+#endif
  plug_set_family(desc, EN_FAMILY, NULL);
  
  plug_set_description(desc, FR_DESC, "francais");
@@ -112,9 +113,9 @@ register_service(desc, port, proto)
      const char		*proto;
 {
   char	k[96];
-  int	l;
 
 #ifdef DEBUG
+  int	l;
   if (port < 0 || proto == NULL ||
       (l = strlen(proto)) == 0 || l > sizeof(k) - 10)
     {
@@ -390,12 +391,9 @@ mark_ssh_server(desc, port, buffer, trp)
  char * buffer;
  int trp;
 {
- char ban[512];
  register_service(desc, port, "ssh");
  while((buffer[strlen(buffer)-1]=='\n')||
        (buffer[strlen(buffer)-1]=='\r'))buffer[strlen(buffer)-1]='\0';
- snprintf(ban, sizeof(ban), "ssh/banner/%d", port);
- plug_set_key(desc, ban, ARG_STRING, buffer);
  post_note(desc, port, "An ssh server is running on this port");
 }
 
@@ -640,7 +638,7 @@ mark_finger_server(desc, port, banner, trp)
      const char		*banner;
      int		port, trp;
 {
-  char	tmp[256], *p;
+  char	tmp[256];
 
 
   register_service(desc, port, "finger");
@@ -748,8 +746,7 @@ mark_time_server(desc, port, banner, trp)
      const char		*banner;
      int		port, trp;
 {
-  char	tmp[256], *p;
-  int	l;
+  char	tmp[256];
 
   register_service(desc, port, "time");  
   snprintf(tmp, sizeof(tmp), "A time server seems to be running on this port%s",
@@ -1045,33 +1042,6 @@ mark_adsgone(desc, port, buffer, trp)
 
 
 
-/*
- * Sig by "Peter C. Parrish" <parrish@ukw.com>
- *
- * unknown (5555/tcp)|11154|Security Note|An unknown server is running on this
- * port.\nIf you know what it is, please send this banner to the Nessus team:
- * 00: 48 50 20 4f 70 65 6e 56 69 65 77 20 53 74 6f 72    HP OpenView Stor
- * 61 67 65 20 44 61 74 61 20 50 72 6f 74 65 63 74    age Data Protect
- * 6f 72 20 41 2e 30 35 2e 30 30 3a 20 49 4e 45 54    or A.05.00: INET
- * 2c 20 69 6e 74 65 72 6e 61 6c 20 62 75 69 6c 64    , internal build
- * 20 31 39 30 2c 20 62 75 69 6c 74 20 6f 6e 20 54     190, built on T
- * 75 65 20 4a 75 6c 20 31 36 20 31 37 3a 33 37 3a    ue Jul 16 17:37:
- * 33 32 20 32 30 30 32 0a                            32 2002.
- */
-static void
-mark_hpov_storage(desc, port, buffer, trp)
- struct arglist * desc;
- int port, trp;
- char * buffer;
-{
- char ban[255];
- register_service(desc, port, "hpov-storage");
- plug_set_key(desc, ban, ARG_STRING, buffer);
- snprintf(ban, sizeof(ban), "HP OpenView Storage Data Protector%s", get_encaps_through(trp));
- post_note(desc, port, ban);
-}
-
-
 /* Sig from  harm vos <h.vos@fwn.rug.nl> :
  * 
  * 00: 2a 20 41 43 41 50 20 28 49 4d 50 4c 45 4d 45 4e    * ACAP (IMPLEMEN
@@ -1608,8 +1578,8 @@ mark_pnsclient(desc, port, buffer, trp)
 
  register_service(desc, port, "pNSClient");
  
- snprintf(ban, sizeof(ban), "A Netsaint plugin (pNSClient.exe) is running on this port%s:",
-	     get_encaps_through(trp), buffer);
+ snprintf(ban, sizeof(ban), "A Netsaint plugin (pNSClient.exe) is running on this port%s",
+	     get_encaps_through(trp));
  post_note(desc, port, ban);	 
 }
 
@@ -1623,8 +1593,8 @@ mark_veritas_backup(desc, port, buffer, trp)
  char ban[512];
  register_service(desc, port, "VeritasNetBackup");
  
- snprintf(ban, sizeof(ban), "VeritasNetBackup is running on this port%s:",
-	     get_encaps_through(trp), buffer);
+ snprintf(ban, sizeof(ban), "VeritasNetBackup is running on this port%s",
+	     get_encaps_through(trp));
  post_note(desc, port, ban);	 
 }
 
@@ -1802,9 +1772,6 @@ setNonblocking(int fd)
 #endif
 
 
-
-
-
 static int plugin_do_run(desc, h, test_ssl)
  struct arglist * desc;
  struct arglist * h;
@@ -1814,7 +1781,6 @@ static int plugin_do_run(desc, h, test_ssl)
  u_short unknown[65535];
  int num_unknown = 0;
  int len_head = strlen(head);
- int	might_be_socks = 0;
  
  int	rw_timeout = 5, cnx_timeout = 5, wrap_timeout = 3;
  int	x, timeout;
@@ -1825,10 +1791,9 @@ static int plugin_do_run(desc, h, test_ssl)
 #endif
   unsigned char	*p;
   time_t	t1, t2;
-  int		delta_t;
+  int		delta_t = 0;
   fd_set	rfds, wfds, xfds;
   struct timeval tv;
-  
 
   if (rw_timeout_s != NULL && (x = atoi(rw_timeout_s)) > 0)
     rw_timeout = x;
@@ -1857,18 +1822,45 @@ static int plugin_do_run(desc, h, test_ssl)
     int port = atoi(h->name+len_head);
     int flg = 0;
     int	unindentified_service = 0;
+    int three_digits = 0;
     int maybe_wrapped = 0;
+    char	kb[64];
+    int		get_sent = 0;
+    int		type;
 
-    if (test_ssl == 2 || 
-	test_ssl == 1 && known_ssl_port(port))
-      cnx = open_stream_connection_unknown_encaps(desc, port, cnx_timeout, &trp);
+
+    sprintf(kb, "Banner/%d", port);
+#ifdef NEW_KB_MGMT
+    banner = plug_get_key(desc, kb, &type);
+#else
+    banner = plug_get_key(desc, kb);
+#endif
+
+    if (type == ARG_STRING  && banner != NULL && banner[0] != '\0' /* strlen > 0*/)
+      {
+#ifdef DEBUG
+	fprintf(stderr, "Banner is known on port %d - will not open a new connection\n", port);
+#endif
+	cnx = -1;
+	trp = NESSUS_ENCAPS_IP;
+      }
     else
       {
-	trp = NESSUS_ENCAPS_IP;
-	cnx = open_stream_connection(desc, port, trp, cnx_timeout);
+#ifdef DEBUG
+	fprintf(stderr, "Banner is unknown on port %d - connecting...\n", port);
+#endif
+	banner = NULL;
+	if (test_ssl == 2 || 
+	    (test_ssl == 1 && known_ssl_port(port)))
+	  cnx = open_stream_connection_unknown_encaps(desc, port, cnx_timeout, &trp);
+	else
+	  {
+	    trp = NESSUS_ENCAPS_IP;
+	    cnx = open_stream_connection(desc, port, trp, cnx_timeout);
+	  }
       }
   
-  if(cnx >= 0)
+  if(cnx >= 0 || banner != NULL)
    {
     int		len;
     int		realfd = nessus_get_socket_from_connection(cnx);
@@ -1894,7 +1886,17 @@ static int plugin_do_run(desc, h, test_ssl)
 #define HTTP_GET	"GET / HTTP/1.0\r\n\r\n"
 
     len = 0; timeout = 0;
-    bzero(buffer, sizeof(buffer));
+    if (banner != NULL)
+      {
+	len = strlen(banner);
+	if (len >= sizeof(buffer))
+	  len = sizeof(buffer) -1;
+	memcpy(buffer, banner, len);
+	buffer[len] = '\0';
+      }
+    else
+      {
+      bzero(buffer, sizeof(buffer));
 #ifdef SMART_TCP_RW
     if (trp == NESSUS_ENCAPS_IP && realfd >= 0)
       {
@@ -1928,10 +1930,20 @@ select_again:
       {
 	write_stream_connection(cnx, HTTP_GET, sizeof(HTTP_GET)-1);
 	t1 = time(NULL);
+	get_sent = 1;
 	len = read_stream_connection(cnx, buffer, sizeof(buffer)-1);
+#if 1
+	/* Try to work around broken web server (or "magic read" bug??) */
+	if (len > 0 && len < 8 && strncmp(buffer, "HTTP/1.", len) == 0)
+	  {
+	    int	len2 = read_stream_connection(cnx, buffer+len, sizeof(buffer)-1-len);
+	    if (len2 > 0) len += len2;
+	  }
+#endif
 	t2 = time(NULL);
 	delta_t = t2 - t1;
       }
+      }
 
    if (len > 0)
      {
@@ -1944,12 +1956,26 @@ select_again:
 	t[0]='\0';
 	}
 	
+   if (isdigit(banner[0]) && isdigit(banner[1]) && isdigit(banner[2]) &&
+       (banner[3] == '\0' || isspace(banner[3]) || banner[3] == '-'))
+     {
+       plug_set_key(desc, "Services/three_digits", ARG_INT, (void*) port);
+       /* Do *not* set Known/tcp/<port> to "three_digits": the service must
+	* remain "unknown" */
+       three_digits = 1;
+     }
+
+   if (get_sent)
+     sprintf(kb, "FindService/tcp/%d/get_http", port);
+   else
+     sprintf(kb, "FindService/tcp/%d/spontaneous", port);
+   plug_set_key(desc, kb, ARG_STRING, banner);
+
    origline = estrdup(banner);
    if( strchr(origline, '\n') != NULL ){
    	char * t = strchr(origline, '\n');
 	t[0] = '\0';
 	}
-    
     /*
      * Many services run on the top of an HTTP protocol,
      * so the HTTP test is not an 'ELSE ... IF'
@@ -1994,7 +2020,7 @@ select_again:
         mark_shoutcast_server(desc, port, origline, trp);
     else  if((strstr(line, "smtp") || strstr(line, "simple mail transfer") || strstr(line, "mail server") || strstr(line, "messaging") || strstr(line, "Weasel")) && !strncmp(line, "220", 3))
     	mark_smtp_server(desc, port, origline, trp);
-    else if(strstr(line, "220 ***************")) /* CISCO SMTP (?) - see bug #175 */
+    else if(strstr(line, "220 ***************") || strstr(line, "220 eSafe@")) /* CISCO SMTP (?) - see bug #175 */
         mark_smtp_server(desc, port, origline, trp);
     else if(strstr(line, "220 esafealert") != NULL )
     	mark_smtp_server(desc, port, origline, trp);
@@ -2025,7 +2051,7 @@ select_again:
         mark_time_server(desc, port, banner, trp);
     else if(strstr(buffer, "rmserver")||strstr(buffer, "realserver"))
         mark_rmserver(desc, port, origline, trp);
-    else if((strstr(line, "ftp") || strstr(line, "netpresenz") || strstr(line, "serv-u") || strstr(line, "service ready for new user")) && !strncmp(line, "220", 3))
+    else if((strstr(line, "ftp") || strstr(line, "winsock") || strstr(line, "axis network camera") || strstr(line, "netpresenz") || strstr(line, "serv-u") || strstr(line, "service ready for new user")) && !strncmp(line, "220", 3))
     	mark_ftp_server(desc, port, origline, trp);	
     else if(strncmp(line, "220-", 4) == 0) 	/* FTP server with a long banner */
     	mark_ftp_server(desc, port, NULL, trp);	
@@ -2142,8 +2168,6 @@ select_again:
     	mark_pop3pw_server(desc, port, origline, trp);
     else if(strstr(line, "welcome!psybnc@") != NULL)
     	mark_psybnc(desc, port, origline, trp);
-    else if(strstr(line, "hp openview storage protect") != NULL)
-	mark_hpov_storage(desc, port, origline, trp);
     else if(strncmp(line, "* acap ", strlen("* acap ")) == 0)
       	mark_acap_server(desc, port, origline, trp);
     else if(strstr(origline, "Sorry, you (") != NULL &&
@@ -2217,7 +2241,7 @@ select_again:
 	    strstr(banner, "Line     User") ||
 	    strstr(banner, "Login name: GET"))
       {
-	char	c;
+	char	c = '\0';
 	if(p != NULL)
 	{
 	while (p - banner > 0 && isspace(*p))
@@ -2251,7 +2275,8 @@ select_again:
 #endif
 	 maybe_wrapped = 1;
    }
-   close_stream_connection(cnx);
+   if (cnx > 0)
+     close_stream_connection(cnx);
 
 #ifdef DETECT_WRAPPED_SVC
    /*
@@ -2263,7 +2288,6 @@ select_again:
 
    if (maybe_wrapped /*&& trp == NESSUS_ENCAPS_IP && wrap_timeout > 0*/)
      {
-       time_t	t;
        int	nfd, fd, x, flag = 0;
        char	b;
 
@@ -2301,24 +2325,12 @@ select_again2:
               }
 	   else if (x > 0)
 	     {
-recv_again:             
 	       errno = 0;
 #ifdef MSG_DONTWAIT	       
 	       x = recv(fd, &b, 1, MSG_DONTWAIT);
 #else
 	       x = recv(fd, &b, 1, 0);
 #endif
-		 
-#if 0
-	       fprintf(stderr, "recv(port=%d)=%d\n", port, x);
-
-	       if (x < 0)
-                {
-                 if ( errno == EINTR ) goto recv_again;
-		 perror("recv");
-                }
-		 
-#endif
 
 		 
 	       if (x == 0 || (x < 0 && errno == EPIPE))
@@ -2369,9 +2381,10 @@ and in %d when we just wait. It is  probably not wrapped\n",
    if (unindentified_service && port != 139)
      /* port 139 can't be marked as 'unknown' */
     {
-      char	ban[256];
       unknown[num_unknown++] = port;
-      mark_unknown_svc(desc, port, banner, trp);
+      /* find_service_3digits will run after us */
+      if (! three_digits)
+	mark_unknown_svc(desc, port, banner, trp);
     }
    efree(&banner); 
    }
@@ -2411,6 +2424,30 @@ static void sigchld(int s)
  }
 }
 
+#ifdef NEW_KB_MGMT /* In Nessus 2.1.x and newer, the KB items are preceeded by their length */
+static int fwd_data(int in, int out, pid_t sender)
+{
+ int e;
+ static char * buf = NULL;
+ static int    bufsz = 0;
+ int type;
+
+
+
+ e = internal_recv(in, &buf, &bufsz, &type);
+ if ( e <= 0 ) return -1;
+ e = internal_send(out, buf, type);
+
+ if ( bufsz > 65535 )
+  {
+  efree(&buf);
+  buf = NULL;
+  bufsz = 0;
+  }
+  
+ return 0;
+}
+#else
 static int fwd_data(int in, int out, pid_t sender)
 {
  fd_set rd;
@@ -2419,11 +2456,12 @@ static int fwd_data(int in, int out, pid_t sender)
  
  for(;;)
  {
+ int err;
  tv.tv_sec = 0;
- tv.tv_usec = 1000;
+ tv.tv_usec = 10000;
  FD_ZERO(&rd);
  FD_SET(in, &rd);
- if(select(in + 1, &rd, NULL, NULL, &tv) > 0)
+ if((err = select(in + 1, &rd, NULL, NULL, &tv)) > 0)
  {
   int n = recv_line(in, buf, sizeof(buf));
   int m = 0, e;
@@ -2433,26 +2471,34 @@ static int fwd_data(int in, int out, pid_t sender)
    {
     while ( m != n )
     {
+     errno = 0;
      e = send(out, buf + m, n - m , 0);
-     if( e <= 0 )
+     if( e <= 0 && errno != EINTR )
       return -1;
-     else
-      m += e;
+     else 
+      if ( e > 0 ) m += e;
     }
    }
  }
  else {
+    if ( err == 0 ) return 0;
     if ( errno == EINTR )errno = 0;
     else break;
     }
  }
  return 0; 
 }
+#endif
 
 int plugin_run(desc)
  struct arglist * desc;
 {
+#ifdef NEW_KB_MGMT
+ struct arglist * h = plug_get_oldstyle_kb(desc);
+#else
  struct arglist * h =  arg_get_value(desc, "key");
+#endif
+
  struct arglist * ag;
  struct arglist * sons_args[MAX_SONS];
  int sons_pipe[MAX_SONS][2];
@@ -2462,7 +2508,12 @@ int plugin_run(desc)
  int port_per_son;
  int i;
  char * head = "Ports/tcp/";
+ struct arglist * globals = arg_get_value(desc, "globals");
+#ifdef NEW_KB_MGMT
+ int one_true_pipe = (int)arg_get_value(globals, "global_socket");
+#else
  int one_true_pipe = (int)arg_get_value(desc, "pipe");
+#endif
  int	test_ssl = 0;
  #ifdef HAVE_SSL
   char * key   = get_plugin_preference(desc, KEY_FILE);
@@ -2484,10 +2535,12 @@ int plugin_run(desc)
   
   test_ssl = 2;
   if (test_ssl_s != NULL)
+    {
     if (strcmp(test_ssl_s, "None") == 0)
       test_ssl = 0;
     else if (strcmp(test_ssl_s, "Known SSL ports") == 0)
       test_ssl = 1;
+    }
 
   if(key || cert)
   {
@@ -2598,10 +2651,22 @@ int plugin_run(desc)
    sons[i] = fork();
    if(sons[i] == 0)
    {
-    int old = (int)arg_get_value(desc, "pipe");
-    close(old);
-    arg_set_value(desc, "pipe", -1, (void*)sons_pipe[i][0]);
+#ifdef NEW_KB_MGMT
+    int soc = (int)arg_get_value(globals, "global_socket");
+#else
+    int soc = (int)arg_get_value(desc, "pipe");
+#endif
     close(sons_pipe[i][1]);
+#ifdef NEW_KB_MGMT
+    close(soc);
+    soc = dup2(sons_pipe[i][0], 4);
+    close(sons_pipe[i][0]);
+    arg_set_value(globals, "global_socket", sizeof(soc), (void*)soc);
+    arg_set_value(desc, "SOCKET", sizeof(soc), (void*)soc);  
+#else
+    close(soc);
+    arg_set_value(desc, "pipe", -1, (void*)sons_pipe[i][0]);
+#endif
     signal(SIGTERM, _exit);
     plugin_do_run(desc, sons_args[i], test_ssl);
     exit(0);
@@ -2632,7 +2697,7 @@ int plugin_run(desc)
   }
   
 again:
-  tv.tv_usec = 5000;
+  tv.tv_usec = 100000;
   tv.tv_sec  = 0;
   e = select(max + 1, &rd, NULL, NULL, &tv);
   if ( e < 0 && errno == EINTR ) goto again;
