@@ -6,6 +6,7 @@
  */
 
 #include <includes.h>
+#include "nasl.h"
 
 /*
  *  Plugin functions
@@ -13,7 +14,7 @@
 
 #define EN_NAME "Nmap"
 #define FR_NAME "Nmap"
-
+#if NASL_LEVEL < 2181
 #define EN_DESC "\
 This plugin calls the program nmap(1). See\n\
 the section 'plugins options' to configure it"
@@ -21,7 +22,11 @@ the section 'plugins options' to configure it"
 #define FR_DESC "\
 Ce plugin appelle le programme nmap(1). Allez voir\n\
 la section 'plugins options' pour le configurer"
-
+#else
+# define EN_DESC "\
+This plugin calls the program nmap(1). \n\
+It is obsolete: you must use the new NASL version"
+#endif
 
 #define COPYRIGHT "Nmap is copyright (C) Fyodor - <fyodor@dhp.com>"
 
@@ -59,8 +64,10 @@ la section 'plugins options' pour le configurer"
 #define IDENTD_INFO "Get Identd info"
 #define IDENTD_DFL "no"
 
+#if 0
 #define PORTRANGE "Port range"
-#define PORTRANGE_DFL "User specified range;Default range (nmap-services + privileged ports);Fast scan (nmap-services)"
+#define PORTRANGE_DFL "User specified range;Fast scan (nmap-services)"
+#endif
 
 
 #define DONT_RANDOMIZE_PORTS "Do not randomize the  order  in  which ports are scanned"
@@ -210,7 +217,7 @@ update_nmap_version()
   int		flag = -1, x;
   static char	*args[] = { NMAP, "-V", NULL };
 
-  if ((fp = nessus_popen(NMAP, (const char**)args, &pid)) == NULL)
+  if ((fp = nessus_popen(NMAP, args, &pid)) == NULL)
     return -1;
   
   while (fgets(str, sizeof(str), fp) != NULL)
@@ -290,7 +297,7 @@ ptycall_nmap
 {
   FILE *fp;
   
-  if ((fp = nessus_popen(cmd, (const char**)argv, pid)) == 0)
+  if ((fp = nessus_popen(cmd, argv, pid)) == 0)
     return NULL;
 
   nmap_pid = *pid;
@@ -306,12 +313,12 @@ read_nmap_output(struct arglist * env,
 {
  int my_host_found = 0;
  struct in_addr * ip;
- char	ip_expected[20];
+ char	ip_expected[32];
  char buf [1024], *s;
  int	num_ports = 0;
 
  ip = plug_get_host_ip(env);
- snprintf(ip_expected, sizeof(ip_expected), "%s", inet_ntoa(*ip));
+ snprintf(ip_expected, sizeof(ip_expected), "%s:", inet_ntoa(*ip));
  
  for(;;) {
    if ((s = fgets (buf, sizeof (buf) - 1, fp)) == NULL)
@@ -368,10 +375,12 @@ read_nmap_output(struct arglist * env,
 
 PlugExport int plugin_init(struct arglist * desc)
 {
+#if NASL_LEVEL < 2181
  if(!is_shell_command_present(NMAP))
+#endif
 	return -1;
  plug_set_id(desc, 10336);
- plug_set_version(desc, "$Revision: 1.103 $");
+ plug_set_version(desc, "$Revision: 1.109 $");
    
  update_nmap_version();
           
@@ -383,13 +392,20 @@ PlugExport int plugin_init(struct arglist * desc)
  plug_set_summary(desc, EN_SUMMARY, NULL);
  
  
+#ifdef FR_DESC
  plug_set_description(desc, FR_DESC, "francais");
+#endif
  plug_set_description(desc, EN_DESC, NULL);
  
  plug_set_copyright(desc, COPYRIGHT,NULL);
+#if NASL_LEVEL < 2181
  plug_set_category(desc, ACT_SCANNER);
  plug_set_family(desc, FR_FAMILY, "francais");
  plug_set_family(desc, EN_FAMILY, NULL);
+#else
+ plug_set_category(desc, ACT_GATHER_INFO);
+ plug_set_family(desc, "Obsolete plugins", NULL);
+#endif
  plug_set_dep(desc, "ping_host.nasl");
  
  /*
@@ -403,6 +419,7 @@ PlugExport int plugin_init(struct arglist * desc)
  /*
   * Nmap options
   */
+#if NASL_LEVEL < 2181
  if (nmap_V_major >= 3 || (nmap_V_major == 2 && nmap_V_minor == 99))
    add_plugin_preference(desc, TCP_PORTSCAN, PREF_RADIO, TCP_DFL_300);
  else
@@ -415,7 +432,6 @@ PlugExport int plugin_init(struct arglist * desc)
  add_plugin_preference(desc, OS_HIDDEN, PREF_CHECKBOX, OS_HIDDEN_DFL);
  add_plugin_preference(desc, FRAG_SCAN, PREF_CHECKBOX, FRAG_DFL);
  add_plugin_preference(desc, IDENTD_INFO, PREF_CHECKBOX, IDENTD_DFL);
- add_plugin_preference(desc, PORTRANGE, PREF_RADIO, PORTRANGE_DFL);
  add_plugin_preference(desc, DONT_RANDOMIZE_PORTS, PREF_CHECKBOX, DONT_RANDOMIZE_PORTS_DFL);
  add_plugin_preference(desc, SOURCE_PORT, PREF_ENTRY, SOURCE_DFL);
 
@@ -434,6 +450,7 @@ PlugExport int plugin_init(struct arglist * desc)
  add_plugin_preference(desc, INITIAL_RTT_TIMEOUT, PREF_ENTRY, INITIAL_RTT_DFL);
  add_plugin_preference(desc, SCAN_DELAY, PREF_ENTRY, SCAN_DELAY_DFL);
  add_plugin_preference(desc, NMAP_FILE, PREF_FILE, NMAP_FILE_DFL);
+#endif
  return(0);
 }
 
@@ -506,7 +523,8 @@ again:
  if(opt !=0 && strcmp(opt, "no") == 0)
   argv = append_argv (argv, "-P0");
  
- opt = get_plugin_preference(desc, PORTRANGE);
+ /*opt = get_plugin_preference(desc, PORTRANGE);*/
+ opt = NULL;
  if (opt == NULL || strncmp (opt, "User spec", 9) == 0) 
    {
      char	*pr = get_preference (desc, "port_range");
@@ -813,8 +831,8 @@ again:
 	goto again;
  	}
  comm_send_status(globals, arg_get_value(hostinfos, "NAME"),"portscan",  100,100);
- plug_set_key (desc, "Host/scanned", ARG_STRING, "1");
- if(num_udp_ports)plug_set_key (desc, "Host/udp_scanned", ARG_STRING, "1");
+ plug_set_key (desc, "Host/scanned", ARG_INT, (void*)1);
+ if(num_udp_ports)plug_set_key (desc, "Host/udp_scanned", ARG_INT, (void*)1);
 
 #ifdef BORING_COMMENTS
  if (! warned && num_ports < 2 * 65535)
