@@ -34,12 +34,12 @@ struct tftp_header {
 #define NAME "TFTP get file"
 #define DESC "\
 The TFTP (Trivial File Transfer Protocol) allows\n\
-remote user to read file withour having to log in.\n\
+remote users to read files without having to log in.\n\
 This may be a big security flaw, especially if tftpd\n\
 (the TFTP server) is not well configured by the\n\
-admin of the remote host\n\
+admin of the remote host.\n\
 Solution : disable it\n\
-Risk factor : high"
+Risk factor : High"
 
 #define FR_DESC "\
 Le TFTP (Trivial File Transfer Protocol) permet\n\
@@ -52,13 +52,14 @@ Solution : dsactivez-le\n\
 Facteur de risque : Elev"
 
 #define COPYRIGHT "no copyright"
-#define SUMM "Attempts to grab a via through tftp"
+#define SUMM "Attempts to grab a file through tftp"
 
 
 PlugExport int plugin_init(struct arglist *desc);
 PlugExport int plugin_init(struct arglist *desc)
 {
   plug_set_id(desc, 10339);
+  plug_set_version(desc, "$Revision: 1.9 $");
   plug_set_cve_id(desc, "CAN-1999-0498");
   plug_set_name(desc, NAME, NULL);
   plug_set_description(desc, FR_DESC, "francais");
@@ -78,18 +79,30 @@ PlugExport int plugin_run(struct arglist * env)
  struct sockaddr_in addr;
  struct in_addr *p = plug_get_host_ip(env);
  struct tftp_header  * packet;
- char * p_packet = emalloc(512);
+ char * p_packet;
  char * test_file = get_preference(env, "test_file");
  char * file = NULL;
  int b;
- int i;
  fd_set read_set;
  char * report;
  int flaw = 0;
  int len = sizeof(struct sockaddr_in);
- struct timeval timeout = {10,0};
+ struct timeval timeout = {5,0};
+ char * asc_to = get_preference(env, "checks_read_timeout");
+ 
+ 
+ if(asc_to != NULL)
+ {
+  int x= atoi(asc_to);
+  if(x > 0 && x < 255)
+   timeout.tv_sec = x;
+ }
+ 
+ if( test_file == NULL )
+ 	test_file = estrdup("/etc/passwd");
+	
+ p_packet = emalloc(512 + strlen(test_file));
  
- if(!test_file)test_file = strdup("/etc/passwd");
  packet = (struct tftp_header *)p_packet;
  
  packet->th_opcode=htons(RRQ);
@@ -98,14 +111,12 @@ PlugExport int plugin_run(struct arglist * env)
  
 
  soc = socket(AF_INET, SOCK_DGRAM, 0);
- addr.sin_addr.s_addr = htonl(INADDR_ANY);
+ set_socket_source_addr(soc, 0);
  addr.sin_family = AF_INET;
- addr.sin_port = 0;
- if(bind(soc, (struct sockaddr *)&addr, len)==-1)return(0);
  addr.sin_addr = *p;
  addr.sin_port = htons(69);
- 
- b = sendto(soc, packet, 22, 0, (struct sockaddr *)&addr, len);
+ connect(soc, (struct sockaddr*)&addr, len);
+ b = send(soc, packet, 22, 0);
  
  addr.sin_addr = *p;
  addr.sin_port = 0;
@@ -120,24 +131,28 @@ PlugExport int plugin_run(struct arglist * env)
  FD_SET(soc, &read_set);
  select(soc+1, &read_set, NULL, NULL, &timeout);
  if(!FD_ISSET(soc, &read_set))break;
- b = recvfrom(soc,packet, 512, 0, (struct sockaddr *)&addr, &len);
+ b = recv(soc,packet, 512, 0);
+ if(b < sizeof(struct tftp_header))exit(0);
  if(ntohs(packet->th_opcode)==3)
  {
   /* We receive some data : there is a flaw */
-  char * tmp = NULL;
+  char tmp[512];
   char * tmp2;
   flaw++;
-  tmp = emalloc(512);
-  sprintf(tmp, "%s", packet->th_msg);
-  if(!file)tmp2 = emalloc(strlen(tmp)+1);
-  else tmp2 = emalloc(strlen(file)+strlen(tmp)+1);
+  snprintf(tmp, sizeof(tmp), "%s", packet->th_msg);
+  if( file == NULL )
+  	tmp2 = emalloc(strlen(tmp)+1);
+  else 
+  	tmp2 = emalloc(strlen(file)+strlen(tmp)+1);
   
-  if(!file)strncpy(tmp2, tmp, strlen(tmp));
-  else sprintf(tmp2, "%s%s", file,tmp);
-  if(file)efree(&file);
-  file = emalloc(strlen(tmp2)+1);
+  if( file == NULL )
+  	strncpy(tmp2, tmp, strlen(tmp));
+  else snprintf(tmp2, strlen(file)+strlen(tmp)+1, "%s%s", file, tmp);
+  
+  if(file != NULL )
+  	efree(&file);
+  file = emalloc( strlen(tmp2) + 1 );
   strncpy(file, tmp2, strlen(tmp2));
-  efree(&tmp);
   efree(&tmp2);
  }
  else break;
@@ -150,14 +165,14 @@ PlugExport int plugin_run(struct arglist * env)
  
  if(flaw)
  {
- report = emalloc(255+strlen(file));
+ report = emalloc(255+strlen(file)+strlen(test_file));
  sprintf(report, "It was possible to retrieve the file %s\n\
 through tftp. Here is what we could grab : \n%s\n\n\
 Solution : disable the tftp daemon, or if you really need it\n\
-run it in a chrooted environnement", test_file, file);
+run it in a chrooted environment", test_file, file);
  efree(&file);
  plug_set_key(env, "tftp/get_file", ARG_INT, (void*)1);
- post_hole_udp(env, 69,report);
+ post_hole_udp(env, 69, report);
  }
  return(0);
 }
